@@ -320,30 +320,116 @@ function systemctl_swift {
     done
 }
 
+# Special case for OVS 2.9 where we need to change the OVS config file
+# to run with the right user
+function change_ovs_2_9_user {
+    local ovs_config_file="/etc/sysconfig/openvswitch"
+
+    if ! grep -q '^OVS_USER_ID="*openvswitch:hugetlbfs"*' $ovs_config_file; then
+        if grep -q "^\#*OVS_USER_ID=" $ovs_config_file; then
+            sed -i 's/^\#*OVS_USER_ID=.*/OVS_USER_ID="openvswitch:hugetlbfs"/' $ovs_config_file
+        else
+            sed -i '$ a OVS_USER_ID="openvswitch:hugetlbfs"' $ovs_config_file
+        fi
+    fi
+}
+
+# Special case for OVS 2.9 where we need to create a one-time service file,
+# that will change any remaining permissions after reboot if needed
+function change_ovs_2_9_perms {
+    local ovs_owner=$(find /etc/openvswitch /var/log/openvswitch ! -user openvswitch ! -group hugetlbfs 2> /dev/null)
+    if [ ! -z "${ovs_owner}" ]; then
+            cat >/etc/systemd/system/multi-user.target.wants/fix-ovs-permissions.service <<EOL
+[Unit]
+Description=One time service to fix permissions in OpenvSwitch
+Before=openvswitch.service
+
+[Service]
+Type=oneshot
+User=root
+ExecStart=/usr/bin/bash -c "/usr/bin/chown -R openvswitch:hugetlbfs /etc/openvswitch /var/log/openvswitch || true"
+ExecStartPost=/usr/bin/rm /etc/systemd/system/multi-user.target.wants/fix-ovs-permissions.service
+TimeoutStartSec=0
+RemainAfterExit=no
+
+[Install]
+WantedBy=default.target
+EOL
+        chmod a+x /etc/systemd/system/multi-user.target.wants/fix-ovs-permissions.service
+    fi
+}
+
 # Special-case OVS for https://bugs.launchpad.net/tripleo/+bug/1635205
 # Update condition and add --notriggerun for +bug/1669714
 function special_case_ovs_upgrade_if_needed {
     # Always ensure yum has full cache
     yum makecache || echo "Yum makecache failed. This can cause failure later on."
-    if rpm -qa | grep "^openvswitch-2.5.0-14" || rpm -q --scripts openvswitch | awk '/postuninstall/,/*/' | grep "systemctl.*try-restart" ; then
-        echo "Manual upgrade of openvswitch - ovs-2.5.0-14 or restart in postun detected"
-        rm -rf OVS_UPGRADE
-        mkdir OVS_UPGRADE && pushd OVS_UPGRADE
-        echo "Attempting to downloading latest openvswitch with yumdownloader"
-        yumdownloader --resolve openvswitch
-        for pkg in $(ls -1 *.rpm);  do
-            if rpm -U --test $pkg 2>&1 | grep "already installed" ; then
-                echo "Looks like newer version of $pkg is already installed, skipping"
-            else
+    # Ovs uses openvswitch:hugetlbfs as user and group settings
+    # when updating to 2.8 onwards, but openvswitch user is not
+    # created during package update. This adds workaround to
+    # make sure openvswitch user exist before running package
+    # update. Details can be found at:
+    # https://bugzilla.redhat.com/show_bug.cgi?id=1559374
+    # 42477 is the kolla hugetlbfs gid value.
+    getent group hugetlbfs >/dev/null || \
+        groupadd hugetlbfs -g 42477 && groupmod -g 42477 hugetlbfs
+    getent passwd openvswitch >/dev/null || \
+        useradd -r -d / -s /sbin/nologin -c "Open vSwitch Daemons" openvswitch
+    usermod -a -G hugetlbfs openvswitch
+
+    # first check if ovs needs upgrade
+    OVS_NEEDS_UPGRADE=$(yum check-upgrade openvswitch | awk '/openvswitch/{print}')
+    if [ -z "${OVS_NEEDS_UPGRADE}" ]; then
+        echo "Looks like newer version of openvswitch is already installed, skipping"
+    else
+        if rpm -qa | grep "^openvswitch-2.5.0-14" || rpm -q --scripts openvswitch | awk '/postuninstall/,/*/' | grep "systemctl.*try-restart" ; then
+            echo "Manual upgrade of openvswitch - ovs-2.5.0-14 or restart in postun detected"
+            rm -rf OVS_UPGRADE
+            mkdir OVS_UPGRADE && pushd OVS_UPGRADE
+            echo "Attempting to downloading latest openvswitch with yumdownloader"
+            yumdownloader --resolve openvswitch
+            for pkg in $(ls -1 *.rpm);  do
                 echo "Updating $pkg with --nopostun --notriggerun"
                 rpm -U --replacepkgs --nopostun --notriggerun $pkg
-            fi
-        done
+                break
+            done
+            popd
+
+        else
+            echo "Skipping manual upgrade of openvswitch - no restart in postun detected. Performing automated upgrade"
+            yum update -y openvswitch
+        fi
+
+        change_ovs_2_9_user
+        change_ovs_2_9_perms
+    fi
+}
+
+function special_case_iptables_services_upgrade_if_needed {
+    # Always ensure yum has full cache
+    yum makecache || echo "Yum makecache failed. This can cause failure later on."
+    # Return 0 when no upgrade is needed
+    if yum check-upgrade iptables-services; then
+        echo "Either iptables-services is not installed or a newer version is already there, skipping workaround."
+    fi
+    if rpm -q --scripts iptables-services | awk '/postuninstall/,/*/' | grep "systemctl.*try-restart" ; then
+        echo "Manual upgrade of iptables-services - restart in postun detected"
+        rm -rf ~/IPTABLES_UPGRADE
+        mkdir -p ~/IPTABLES_UPGRADE && pushd ~/IPTABLES_UPGRADE
+        echo "Attempting to download latest iptables-services with yumdownloader"
+        yumdownloader iptables-services # no deps on purpose.
+        pkg="$(ls -1 iptables-services-*.x86_64.rpm)"
+        if [ -z "${pkg}" ]; then
+            echo "Cannot find a valid package for iptables-services, aborting"
+            exit 1
+        fi
+        echo "Updating iptables-services to $pkg with --nopostun --notriggerun --nodeps"
+        rpm -U --replacepkgs --nopostun --notriggerun --nodeps ./${pkg}
+        systemctl daemon-reload
         popd
     else
-        echo "Skipping manual upgrade of openvswitch - no restart in postun detected"
+        echo "Skipping manual upgrade of iptables-services  - no restart in postun detected"
     fi
-
 }
 
 # update os-net-config before ovs see https://bugs.launchpad.net/tripleo/+bug/1695893
@@ -379,6 +465,7 @@ function update_network() {
     update_os_net_config
     # special case https://bugs.launchpad.net/tripleo/+bug/1635205 +bug/1669714
     special_case_ovs_upgrade_if_needed
+    special_case_iptables_services_upgrade_if_needed
 }
 
 # https://bugs.launchpad.net/tripleo/+bug/1704131 guard against yum update
